USE [AdventureWorks2012]
GO

SET STATISTICS IO ON;
SET STATISTICS TIME ON;
/**
DROP INDEX [IX_Person_LastName_FirstName_MiddleName] ON [Person].[Person]
GO
**/

/**** Always have Clean starting point in Non-Prod ****/

CHECKPOINT;
DBCC DROPCLEANBUFFERS;
DBCC FREEPROCCACHE;

-- Which query has less cost? How do we know?

SELECT *
FROM Person.Person

CHECKPOINT;
DBCC DROPCLEANBUFFERS;
DBCC FREEPROCCACHE;

SELECT FirstName, LastName, MiddleName
FROM Person.Person


SELECT FirstName, LastName
FROM Person.Person
WHERE LastName like 'Smith'

CREATE NONCLUSTERED INDEX [IX_Person_LastName_FirstName_MiddleName] ON [Person].[Person]
(
	[LastName] ASC
)

SELECT FirstName, LastName
FROM Person.Person
WHERE LastName like 'Smith'


DROP INDEX [IX_Person_LastName_FirstName_MiddleName] ON [Person].[Person]
CREATE NONCLUSTERED INDEX [IX_Person_LastName_FirstName_MiddleName] ON [Person].[Person]
(
	[LastName]
)
INCLUDE (	[FirstName] )


SELECT FirstName, LastName
FROM Person.Person
WHERE LastName like 'Smith'

/* Now what do we do? */
SELECT FirstName, LastName, MiddleName
FROM Person.Person
WHERE LastName like 'Smith'


CREATE NONCLUSTERED INDEX [IX_Person_LastName_FirstName_MiddleName2] ON [Person].[Person]
(
	[LastName]
)
INCLUDE (	[FirstName], [MiddleName] )

SELECT FirstName, LastName, MiddleName
FROM Person.Person
WHERE LastName like 'Smith'


/* The second variation of this query finds partial, or duplicate, indexes that share leading key columns,
 e.g. Ix1(col1, col2, col3) and Ix2(col1, col2) would be considered duplicate indexes. 
 This query only examines key columns and does not consider included columns. 
These types of indexes are probable dead indexes walking.

-- Overlapping indxes
*/
with indexcols as
(
select object_id as id, index_id as indid, name,
(select case keyno when 0 then NULL else colid end as [data()]
from sys.sysindexkeys as k
where k.id = i.object_id
and k.indid = i.index_id
order by keyno, colid
for xml path('')) as cols
from sys.indexes as i
)
select
object_schema_name(c1.id) + '.' + object_name(c1.id) as 'table',
c1.name as 'index',
c2.name as 'partialduplicate'
from indexcols as c1
join indexcols as c2
on c1.id = c2.id
and c1.indid < c2.indid
and ((c1.cols like c2.cols + '%' and SUBSTRING(c1.cols,LEN(c2.cols)+1,1) = ' ')
or (c2.cols like c1.cols + '%' and SUBSTRING(c2.cols,LEN(c1.cols)+1,1) = ' ')) ;

/* Fix partical duplicate index */
DROP INDEX [IX_Person_LastName_FirstName_MiddleName] ON [Person].[Person]
DROP INDEX [IX_Person_LastName_FirstName_MiddleName2] ON [Person].[Person]

CREATE NONCLUSTERED INDEX [IX_Person_LastName_FirstName_MiddleName2] ON [Person].[Person]
(
	[LastName]
)
INCLUDE (	[FirstName], [MiddleName] )